#include "escena.h"



Escena::Escena(QWidget *parent)
: QGLWidget(parent)
{
    int cols = 640;
    int rows = 480;
	int deviceID = 0;       // 0 = open default camera
    int apiID = 1;         //cv::CAP_ANY; 

    setFocusPolicy(Qt::StrongFocus);
    timer = new QTimer(this);

    connect(timer, SIGNAL(timeout()), this, SLOT(timerDone()));

    //setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);

    //Initiating camera
    cap = new VideoCapture();
	cap->open( deviceID + apiID ); // open the  camera
	
    if (!cap->isOpened()) { // check if we succeeded
        printf("Error while opening camera device cap(1)\n");
        exit(0);
    }

    IMG = matrix_allocate( rows, cols, sizeof (char));
    if (!IMG) {
        fprintf(stderr, "ERROR: No memory\n");
        exit(0);
    }

    cap->set(CV_CAP_PROP_FRAME_WIDTH, cols);
    cap->set(CV_CAP_PROP_FRAME_HEIGHT, rows);
    // namedWindow("Bordes", 1);

    *(cap) >> frame; // get a new frame from camera
    //imwrite( "imagen.png", frame );

    cv::cvtColor(frame, grayFrame, CV_BGR2GRAY);

    // IMG->rows = grayFrame.rows;
    // IMG->cols = grayFrame.cols;
	// fprintf( stderr, "%d %d\n", grayFrame.rows, grayFrame.cols );
	// fprintf( stderr, "%d %d\n", IMG->rows, IMG->cols );
    IMG->type = 5;
    IMG->element_size = sizeof (char);


    gen_Cube( 1.0, 1.0, 1.0, cube );


    //Set default points
    T1[0].x = 0;    T1[0].y = 0;
    T1[1].x = 320;    T1[1].y = 0;
    T1[2].x = 320;    T1[2].y = 240;
    
    T2[0].x = 640;    T2[0].y = 480;
    T2[1].x = 320;    T2[1].y = 480;
    T2[2].x = 320;    T2[2].y = 240; 


    //Init find_Triangles
    TRIsys_allocate( IMG );

    //Init Matrices K , Rt
    for(int i = 0; i < 16; i++){
        MK[i] = 0.0;
        MRT[i] = 0.0;
    }

    //Not tracking
    tracking = false;

    //Start Camera by default
    initscreen();
}


Escena::~Escena()
{
    
    matrix_free( IMG );
    TRIsys_free();
}



void Escena::paintGL(void) {
    draw_IMG();

    //draw_Triangles();

    //draw_Scene();

    //draw_Triangles();
}

void Escena::draw_IMG() {
    glDisable(GL_DEPTH_TEST);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    //getting image from camera
    *(cap) >> frame; // get a new frame from camera
    //Preprocessing image
    //Thresholding settings
    //int threshold_value = 100;
    //int threshold_type = 1;
    //int const max_BINARY_value = 255;

    //flip(frame, frame, 0);
    cv::cvtColor(frame, grayFrame, CV_BGR2GRAY);
    // GaussianBlur(grayFrame, grayFrame, Size(7, 7), 1.5, 1.5); //Smooth filter
    // threshold(grayFrame, grayFrame, threshold_value, max_BINARY_value, threshold_type);
    // bitwise_not(grayFrame, grayFrame);

    unsigned char b;
    int channel = 1;
    unsigned char *input = (unsigned char*) (grayFrame.data);
    for (int i = 0; i < grayFrame.rows; i++) {
        for (int j = 0; j < grayFrame.cols; j++) {
            b = input[grayFrame.step * i + j * channel ];
            ((byte**) IMG->ptr)[i][j] = b;
        }
    }

    //IMG is the camera frame in graytones, and can be processed after this line

    flip(grayFrame, grayFrame, 0);

    //use fast 4-byte alignment (default anyway) if possible
    glPixelStorei(GL_UNPACK_ALIGNMENT, (grayFrame.step & 3) ? 1 : 4);
    //set length of one complete row in data (doesn't need to equal image.cols)
    // glPixelStorei(GL_UNPACK_ROW_LENGTH, grayFrame.step / grayFrame.elemSize());
    //flip(frame, frame, 0);
    //Drawing color image to the QGL canvas
    // glDrawPixels(frame.cols, frame.rows, GL_BGR, GL_UNSIGNED_BYTE, grayFrame.data);
    glDrawPixels(frame.cols, frame.rows, GL_LUMINANCE, GL_UNSIGNED_BYTE, grayFrame.data);
    glEnable(GL_DEPTH_TEST);




    glPushMatrix();

        glMatrixMode(GL_PROJECTION);
        glLoadIdentity();
        glOrtho ( 0.0, 640.0, 0.0, 480.0, 1 , 51 );
        glMultMatrixd( MK );

        glMatrixMode(GL_MODELVIEW);
        glLoadIdentity();
        glMultMatrixd( MRT );

        glBegin( GL_LINE_LOOP );
            glColor3f(  0.0,  1.0,  0.0 );
            glVertex3f(  0.0,  0.0, 0 );
            glVertex3f(  255.0,  0.0, 0 );
            glVertex3f(  255.0, 255.0, 0 );
            glVertex3f(  0.0,  255.0, 0 );
        glEnd();


        glTranslatef( 255.0/2.0, 255.0/2.0, 50.0 );

        glRotatef( YROT, 0.0, 0.0, 1.0 );

        glScalef( 50.0, 50.0, 50.0);
        draw_Cube( cube );

        

    glPopMatrix();


    

}

// Función que se encarga de dibujar el cubo, con diferentes colores en cada cara.
void Escena::draw_Cube( float cubo[][3] )
{
    static float Mcolores[6][3] = {
        {1.0,0.0,0.0},
        {0.0,1.0,0.0},
        {0.0,0.0,1.0},
        {1.0,1.0,0.0},
        {1.0,1.0,1.0},
        {1.0,0.0,1.0} };

    static int Mcaras[6][4] = {
        { 0, 3, 7, 4},
        { 0, 4, 5, 1},
        { 1, 5, 6, 2},
        { 2, 6, 7, 3},
        { 0, 1, 2, 3},
        { 4, 7, 6, 5} };

    int i;

    glBegin( GL_QUADS );
    for( i=0; i<6; i++ ) {
        glColor3fv( &Mcolores[i][0] );
        glVertex3fv( &cubo[ Mcaras[i][0] ][0] );
        glVertex3fv( &cubo[ Mcaras[i][1] ][0] );
        glVertex3fv( &cubo[ Mcaras[i][2] ][0] );
        glVertex3fv( &cubo[ Mcaras[i][3] ][0] );
    }
    glEnd( );
}


void drawSquare()
{
    glBegin( GL_QUADS );
        glVertex2i( -1 , -1 );
        glVertex2i( 1  , -1 );
        glVertex2i( 1  ,  1 );
        glVertex2i( -1 ,  1 );
    glEnd();
}

void DRAW1()
{   
    float angle = 45;
    int cross = 1;

     //Draw
                    
        //printf("windowWidth = %d \n", windowWidth );
        //printf("windowHeight= %d \n", windowHeight );
        glPushMatrix( );
                    //glTranslatef( windowWidth/2, windowHeight/2 , 0.0 );
                    glTranslatef( 640/2, 480/2 , 0.0 );

                    glRotatef ( 90 , 0.0, 0.0, 1.0);

                    float SS = 25.0;
                    glScalef( SS, SS, SS );

                    glPushMatrix( ); //BIG Square
                        //glRotatef ( angle , 0.0, 0.0, 1.0);
                        //glTranslatef( 1.0, 1.0, 0.0 );
                        glColor3f( 0.4 , 0.4, 0.4);
                        drawSquare( );
                    glPopMatrix( );

                    glColor3f( 0.44 , 0.8, 0.7);
                    glPushMatrix( ); // LittleSquare 1
                        //glColor3f( 1.0 , 0.8, 0.4);
                        glTranslatef(-1.5, 1.5, 0.0);
                        glScalef(0.5, 0.5, 1.0);
                        glRotatef ( angle*cross , 0.0, 0.0, 1.0); //Cross Spin
                        drawSquare( );
                    glPopMatrix( );

                    glPushMatrix( ); // LittleSquare 2
                        //glColor3f( 1.0 , 0.4, 0.4);
                        glTranslatef(1.5, 1.5, 0.0);
                        glScalef(0.5, 0.5, 1.0);
                        glRotatef ( angle , 0.0, 0.0, 1.0);
                        drawSquare( );
                    glPopMatrix( );

                    glPushMatrix( ); // LittleSquare 3
                        //glColor3f( 1.0 , 0.4, 1.0);
                        glTranslatef(-1.5, -1.5, 0.0);
                        glScalef(0.5, 0.5, 1.0);
                        glRotatef ( angle , 0.0, 0.0, 1.0);
                        drawSquare( );
                    glPopMatrix( );

                    glPushMatrix( ); // LittleSquare 4
                        //glColor3f( 1.0 , 0.4, 0.4);
                        glTranslatef(1.5, -1.5, 0.0);
                        glScalef(0.5, 0.5, 1.0);
                        glRotatef ( angle*cross , 0.0, 0.0, 1.0); //Cross Spin
                        drawSquare( );
                    glPopMatrix( );
        glPopMatrix( );
}

void Escena::draw_Triangles() 
{
    /** Clear all pixels **/
    //glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    glViewport (0, 0, (GLsizei) 640, (GLsizei) 480);

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity ();
    glOrtho( 0, 640, 
             480.0, 0.0, 
            -1.0, 1.0 );

      

   //Draw triangle1
    glPushMatrix( );
            //glTranslatef( 640/2, 480/2 , 0.0 );
            //glScalef( SS, SS, SS );
            glLineWidth( 3.5 );
            glBegin(GL_LINE_LOOP); 
                glColor3f(1.0, 0.0, 0.0);
                glVertex3f( T1[0].x,  T1[0].y,  0);
                glVertex3f( T1[1].x,  T1[1].y,  0);
                glVertex3f( T1[2].x,  T1[2].y,  0);
                glVertex3f( T1[0].x,  T1[0].y,  0);
            glEnd();
       
    glPopMatrix( );




    //Draw triangle2
    glPushMatrix( );
        glLineWidth( 3.5 );
        glBegin(GL_LINE_LOOP);
            glColor3f(1.0, 0.0, 0.0);
            glVertex3f( T2[0].x,  T2[0].y,  0);
            glVertex3f( T2[1].x,  T2[1].y,  0);
            glVertex3f( T2[2].x,  T2[2].y,  0);
            glVertex3f( T2[0].x,  T2[0].y,  0);
        glEnd();
    glPopMatrix( );




    //Reset
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
}

void Escena::draw_Scene() 
{   
        
    glPushMatrix();
        glMultMatrixd( MK );
        glMultMatrixd( MRT );
        glTranslatef( 255.0/2.0, 255.0/2.0, -45.0 );
        glScalef( 90.0, 90.0, 90.0);
        draw_Cube( cube );
    glPopMatrix();



    static int Z_NEAR = 4;
    static int Z_FAR = -4;

    //*
    glViewport (0, 0, (GLsizei) 640, (GLsizei) 480);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    //glOrtho ( 0.0, 640.0, 0.0, 480.0, 1, -51 );
    glMultMatrixd( MK );

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

      glPushMatrix();
        glMultMatrixd( MRT );

        glBegin( GL_LINE_LOOP );
        glColor3f(  0.0,  1.0,  0.0 );
            glVertex3f(  0.0,  0.0, 0 );
            glVertex3f(  255.0,  0.0, 0 );
            glVertex3f(  255.0, 255.0, 0 );
            glVertex3f(  0.0,  255.0, 0 );
        glEnd();

        /*
        glBegin( GL_LINE_LOOP );
        glColor3f(  0.0,  1.0,  1.0 );
            glVertex3f(  0.0,  0.0,  0.5 );
            glVertex3f(  255.0,  0.0, 0.5 );
            glVertex3f(  255.0, 255.0, 0.5 );
            glVertex3f(  0.0,  255.0, 0.5 );
        glEnd();
        */

        //glTranslatef( 1/2 , 1/2 , 1/2 );
        //glScalef( 100.0, 100.0, 100.0 );
        

        //glTranslatef( 255/2 , 255/2 , 0.0 );
        //glScalef( 100.0, 100.0, 0.1 );
        draw_Cube( cube );

        


      glPopMatrix();

}


// Función que define al cubo.
void Escena::gen_Cube( float lx, float ly, float lz, float cube[][3] )
{
    float x = lx/2.0;
    float y = ly/2.0;
    float z = lz/2.0;

    /*
    cube[0][0] = 100; cube[0][1] = 100; cube[0][2] = 100;
    cube[1][0] = 100; cube[1][1] = 100; cube[1][2] = 0;
    cube[2][0] = 0; cube[2][1] = 100; cube[2][2] = 0;
    cube[3][0] = 0; cube[3][1] = 100; cube[3][2] = 100;

    cube[4][0] = 100; cube[4][1] = 0; cube[4][2] = 100;
    cube[5][0] = 100; cube[5][1] = 0; cube[5][2] = 0;
    cube[6][0] = 0; cube[6][1] = 0; cube[6][2] = 0;
    cube[7][0] = 0; cube[7][1] = 0; cube[7][2] = 100;
    */
    
    cube[0][0] = x; cube[0][1] = y; cube[0][2] = z;
    cube[1][0] = x; cube[1][1] = y; cube[1][2] =-z;
    cube[2][0] =-x; cube[2][1] = y; cube[2][2] =-z;
    cube[3][0] =-x; cube[3][1] = y; cube[3][2] = z;

    cube[4][0] = x; cube[4][1] =-y; cube[4][2] = z;
    cube[5][0] = x; cube[5][1] =-y; cube[5][2] =-z;
    cube[6][0] =-x; cube[6][1] =-y; cube[6][2] =-z;
    cube[7][0] =-x; cube[7][1] =-y; cube[7][2] = z;
    
}


void Escena::mouseMoveEvent(QMouseEvent * e) {
    //Not implemented
    return;
}

void Escena::keyPressEvent(QKeyEvent * e) {
    //Not implemented
    return;
}

//
// This method puts the initial coordinates each time window is resized
//

void Escena::resizeGL(int l, int a) {
    windowWidth = l;
    windowHeight = a;
    glViewport(0, 0, (GLsizei) l, (GLsizei) a);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
        glOrtho ( 0.0, 640.0, 0.0, 480.0, 1, 51 );

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();
    //emit changeSize();
}


QSizePolicy Escena::sizePolicy() const {
    return QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
}


void Escena::initializeGL() {
    glClearColor(0.8, 0.8, 0.8, 0.0); // Background to a grey tone
}

//--------------------------------------
//--------------------------------------
void Escena::timerDone() {
    YROT += 5.0;
    if( YROT > 360.0 ) YROT = 0.0;


    if(tracking)
        find_TRIANGLES( IMG , T1 ,T2 , &OBJCount, MK , MRT, false);

    emit OBJCounterChanged();
    updateGL();
}


void Escena::capture()
{
    write_pnm( IMG, "inputIMG.pgm" );
    printf("CAPTURED!\n");

    //smoothIMG( IMG );
    //smoothIMG( IMG );
    //smoothIMG( IMG );

    //write_pnm( IMG, "inputIMGS.pgm" );

    find_TRIANGLES( IMG , T1 ,T2 , &OBJCount, MK , MRT,  true);

    printf(" \n MK: \n");
        for (int i = 0; i < 16; ++i)
        {
            if(i%4 == 0) printf("\n");
            printf(" [%d]= %f ", i , MK[i] );
        }

    printf("\n\n MRt: \n");
        for (int i = 0; i < 16; ++i)
        {
            if(i%4 == 0) printf("\n");
            printf(" [%d]= %f ", i , MRT[i] );
        }

    printf("\n DONE -\n");
   



    //Send message to GUI
    emit OBJCounterChanged();

    updateGL();
}





