#ifndef TRIANGLE_H
#define TRIANGLE_H 03

#include <QWidget>
#include <QtOpenGL>
#include <QTimer>
#include <qpainter.h>
#include <ctype.h>
#include "opencv2/opencv.hpp"


//Includes C code files
extern "C" {
#include "matrix.h"
#include "pnm.h"
#include "imgUtils.h"
#include "homography.h"
}

using namespace cv;
class Escena : public QGLWidget {
private:
    //Opencv frames
    VideoCapture *cap;
    Mat frame;
    Mat grayFrame;
    MATRIX *IMG;
    Q_OBJECT // must include this if you use Qt signals/slots
public:
    Escena(QWidget *parent = 0);
    ~Escena();
    QSizePolicy sizePolicy() const;


    void getOBJCount(int *C ) {
        *C = OBJCount;
    }

    void draw_IMG();
    void draw_Triangles();
    void draw_Scene();

    void gen_Cube( float lx, float ly, float lz, float cube[][3] );
    void draw_Cube( float cube [][3]);

    //void setClass(int classM ){
    //    currentClass = classM;
    //}

public slots:
    void timerDone(void);

    void initscreen(void) {
        timer->start(33);
    }

    void stop(void) {
        timer->stop();
    }

    void initTRACK(void) {
        tracking = true;
    }

    void stopTRACK(void) {
        
        //Reset positions
        for (int i = 0; i < 3; ++i)
        {
            T1[i].x = 0;    T1[i].y = 0;
            T2[i].x = 0;    T2[i].y = 0;
        }

        tracking = false;
    }

    
    void capture();


//******************************
signals:
    void OBJCounterChanged(); 

protected:
    void initializeGL(void);
    void resizeGL(int w, int h);
    void paintGL(void);
    void mouseMoveEvent(QMouseEvent *);
    void keyPressEvent(QKeyEvent * e);

private:
    QTimer *timer;
    int windowWidth, windowHeight; // canvas width and height
    float yrot, xrot; // Encene rotation
    int xmouse, ymouse; // Mouse pointer coordinates

    int tracking;
    int OBJCount;

    POINT T1[3], T2[3]; //Triangles point array
    double MK[16], MRT[16]; //K and Rt Matrices

    double YROT;


    float cube[8][3];
 

};


#endif // CUBO_H
