/*
	Program that finds vertex of triangles in an image

	Sergio Herrera July/2018
*/

#include "homography.h"

//Variables
MATRIX *IMG, *FIL;

//Smooth window
int **fil;

int Umbral = 13;

int X,Y;
int OBJcount = 0;
int OBJ[5];
int COLOR;

int Counter = 0;


DPOINT *PointS, *dv;
POINT *perimeter, *vertices; 

byte *pimg;

//Vars for Homography

MATRIX * MT1, *MT2;
double ** T1, **T2;

double medianX , medianY ;		
double varX, varY;
//Build Matrix B = [A|b]
MATRIX *MB,*MR;
double **R, **B;
double * H;
MATRIX * Mres, *MHn;
double ** Hn, **r;
MATRIX *MPX ,*MPR;
double ** PX ,** pRes;

MATRIX* MK , *MKi;
double ** K , ** Ki;
MATRIX* MG;
double ** G;
MATRIX *MRd , *MUU , *MD , *MVT ;
double **Rd,**UU,**D,**VT;
//[R|t]
MATRIX *MRT ,*MM;
double ** RT , ** M ;


int TRIsys_allocate(MATRIX* IN)
{		
	printf("#Allocating resources.... \n");


	//Output IMG
	IMG = matrix_allocate( IN->rows, IN->cols, sizeof(byte) );

	//Smooth Window Matrix
	FIL = matrix_allocate( 3, 3, sizeof( int ) );
	if ( FIL==NULL ) {
		fprintf( stderr, "ERROR allocating memory\n" );
		return -2;
	}
	//Smooth Window
	fil = (int **)FIL->ptr;
	fil[0][0] = 1; fil[0][1] = 1; fil[0][2] = 1;
	fil[1][0] = 1; fil[1][1] = 1; fil[1][2] = 1;
	fil[2][0] = 1; fil[2][1] = 1; fil[2][2] = 1;

	//Triangles Vertices
	PointS = malloc( sizeof(DPOINT) * 3 * (2) );

	//Allocate memory enough for ANY perimeter segment
	perimeter = malloc( sizeof(POINT) * 2000 );
	//Allocate memory for each triangle vertices
	vertices = malloc( sizeof(POINT) * 3 );
	dv = malloc( sizeof(DPOINT) * 3 );



	//Resources for Homography

		//Matrix T1 and T2
		MT1 = matrix_allocate( 3, 3, sizeof(double) );
		MT2 = matrix_allocate( 3, 3, sizeof(double) );
		if( MT1==NULL || MT2==NULL ) {
			fprintf( stderr, "ERROR no hay memoria para la matrices T1 y T1\n" );
			return( -3 );
		}
		T1 = (double **)MT1->ptr;
		T2 = (double **)MT2->ptr;


		//Build Matrix B = [A|b]
		MB = matrix_allocate( 12, 9, sizeof(double) );
		MR = matrix_allocate( 9, 9, sizeof(double) );
		if( MB==NULL || MR==NULL ) {
			fprintf( stderr, "ERROR no hay memoria para la matrices MB y MR\n" );
			return( -4 );
		}
		B = (double **)MB->ptr;
		R = (double **)MR->ptr;
		
		//Homography Matrix
		H = (double *)malloc( 12*sizeof(double) );

		Mres = matrix_allocate( 3, 3, sizeof(double) );
		MHn = matrix_allocate( 3, 3, sizeof(double) );
		if( Mres==NULL || MHn==NULL ) {
			fprintf( stderr, "ERROR no hay memoria para la matrices T1 y T1\n" );
			return( -5 );
		}
		r = (double **)Mres->ptr;
		Hn = (double **)MHn->ptr;


		MPX = matrix_allocate( 4, 1, sizeof(double) );
		MPR = matrix_allocate( 3, 1, sizeof(double) );
		if( MPX == NULL || MPR == NULL ) {
			fprintf( stderr, "ERROR no hay memoria para la matrices [MPX/MPR] \n" );
			return( -15 );
		}
		PX = (double **)MPX->ptr;
		pRes = (double **)MPR->ptr;


		MK = matrix_allocate( 3, 3, sizeof(double) );
		MKi = matrix_allocate( 3, 3, sizeof(double) );
		if( MK == NULL || MKi == NULL ) {
			fprintf( stderr, "ERROR no hay memoria para la matrices MK y MKi\n" );
			return( -7 );
		}
		K = (double **)MK->ptr;
		Ki = (double **)MKi->ptr;


		MG = matrix_allocate( 3, 3, sizeof(double) );
		if( MG == NULL ) {
			fprintf( stderr, "ERROR no hay memoria para la matrices MG \n" );
			return( -7 );
		}
		G = (double **)MG->ptr;

		//Computing r1 x r2
		MRd = matrix_allocate( 3, 3, sizeof(double) );
		MUU = matrix_allocate( 3, 3, sizeof(double) );
		MD = matrix_allocate( 3, 1, sizeof(double) );
		MVT = matrix_allocate( 3, 3, sizeof(double) );
		if( MRd == NULL || MUU == NULL || MD == NULL || MVT == NULL) {
			fprintf( stderr, "ERROR no hay memoria para la matrices UDV^T \n" );
			return( -11 );
		}
		Rd = (double **)MRd->ptr;
		UU = (double **)MUU->ptr;
		D = (double **)MD->ptr;
		VT = (double **)MVT->ptr;

		//Building augmented matrix [R|t]
		MRT = matrix_allocate( 3, 4, sizeof(double) );
		MM = matrix_allocate( 3, 4, sizeof(double) );
		if( MRT == NULL || MM == NULL) {
			fprintf( stderr, "ERROR no hay memoria para la matrices [R|t] \n" );
			return( -14 );
		}
		RT = (double **)MRT->ptr;
		M = (double **)MM->ptr;


			
	return 0;
}

void TRIsys_free()
{
	printf("# Freeing resources.... \n");



	matrix_free( MPX ); matrix_free( MPR );

	matrix_free( MRT );	matrix_free( MM );

	matrix_free( MUU );matrix_free( MD );matrix_free( MVT );
	matrix_free( MG );  matrix_free( MRd );
	matrix_free( MK );  matrix_free( MKi );
	matrix_free( MHn );	matrix_free( Mres );

	matrix_free( MT1 );
	matrix_free( MT2 );
	matrix_free( MB );
	matrix_free( MR );

	free(H);

	free(vertices); free(dv);
	free(perimeter);
	free(PointS);

	matrix_free( IMG );
	matrix_free( FIL );
}



/*
 * Normaliza los datos de entrada
 * */
void normalizar( double S[][2], double E[][2], double vd[4] )
{
  int n = 6;
  double sum1, sum2, prom, var, sig;
  int i;

  // normalizamos x
  sum1 = 0.0;
  sum2 = 0.0;
  i = 0;
  while (i < n){
    sum1 += E[i][0];
    sum2 += E[i][0]*E[i][0];
    i++;
  }
  prom = sum1/n;
  var = sum2/n - prom*prom;
  sig = sqrt(var);
  vd[0] = prom;
  vd[1] = sig;

  i = 0;
  while (i < n){
    S[i][0] = (E[i][0]-prom)/sig;
    i++;
  }

  // normalizamos y
  sum1 = 0.0;
  sum2 = 0.0;
  i = 0;
  while (i < n){
    sum1 += E[i][1];
    sum2 += E[i][1]*E[i][1];
    i++;
  }

  prom = sum1/n;
  var = sum2/n - prom*prom;
  sig = sqrt(var);
  vd[2] = prom;
  vd[3] = sig;

  i = 0;
  while (i < n){
    S[i][1] = (E[i][1]-prom)/sig;
    i++;
  }
}

/**
  Subrutina que calcula descomposición QR
  A = QR, A es de tamaño 8 x 9
  No retorna el valor de  Q
**/
void QR_noQ( double A[][9], double R[][9] )
{
	int i, j, k;
	double sum, tmp;
	int m=8;
	int n=9;
	double V[8][9], vec[8];

    for (i = 0; i < 8; i++)
        for (k = 0; k < 9; k++){
            V[i][k] = A[i][k];
        }

	for( k=0; k<n; k++ ) {

		 sum = 0.0;
		 for( i=0; i<m; i++ ) {
			tmp = V[i][k];
			sum += tmp * tmp;
		 }
		 sum = sqrt( sum );
		 R[k][k] = sum;
		 for( i=0; i<m; i++ )
			vec[i] = V[i][k] / sum;

		 for( j=k+1; j<n; j++ ) {
			sum = 0.0;
			for( i=0; i<m; i++ )
				sum += vec[i] * V[i][j];

				R[k][j] = sum;

			for( i=0; i<m; i++ )
				V[i][j] -= vec[i] * sum;
		 }
	}
}



/* 
   Realiza C = A * B^T
*/
void multiplicaABT(double A[][3], double B[][3], double C[][3])
{
	int i, j;
	for (i=0; i<3;i++)
		for (j=0;j<3;j++)
			C[i][j] = A[i][0]*B[j][0] + A[i][1]*B[j][1] + A[i][2]*B[j][2];
}

// B = A
void CopiaMatriz(double A[3][3], double B[3][3]) {
  int i, j;
  for (i = 0; i < 3; i++) {
    for (j = 0; j < 3; j++) {
      B[i][j] = A[i][j];
    }
  }
}

/*
 * Calcula la matriz de rotación haciendo la descomposición
 *  SVD de la matríz de entrada (a través de la eigendescomposición)
 * */
void CalculaR( double A[][3], double R[][3]  )
{	

    // int i , j;
    // double S[3][3];
	  // double U[3][3], V[3][3];
    // double roots[3];
    //
    // multiplicaATB( A, A, S );
    // dsyevv3( S, V, roots );
    // for( i=0; i<3; i++ )
		// roots[i] = 1.0/sqrt(roots[i]);
    //
    // multiplicaAB( A, V, U );
	  // for( i=0; i<3; i++ )
		//   for( j=0; j<3; j++ )
		// 	   U[i][j] *= roots[j];
    //
    // multiplicaABT( U, V, R );

    double U[3][3];
    double V[3][3];
    double singular_values[3];
    double* dummy_array;
  	int err;

    dummy_array = (double*) malloc(3 * sizeof(double));
    if (dummy_array == NULL) {
		    printf(" No memory available\n");
		    exit(0);
    }

	        printf(" HERE INSIDE !!! \n");

    err = Singular_Value_Decomposition((double*) A, 3, 3, (double*) U, singular_values, (double*) V, dummy_array);
	  free(dummy_array);
    if (err < 0){
		    printf(" Failed to converge\n");
        exit(1);
    }

    multiplicaABT( U, V, R );
  	// for (i = 0; i < 3; ++i) {
  	// 	for (j = 0; j < 3; ++j) {
  	// 		suma = 0.0;
  	// 		for (int k = 0; k < 3; ++k) {
  	// 			suma += U[i][k]*V[j][k];
  	// 		}
  	// 		R[i][j] = suma;
  	// 	}
  	// }
}


/* 
   Realiza C = A * B
*/
void multiplicaAB(double A[][3], double B[][3], double C[][3])
{
	int i, j;
	for (i=0; i<3;i++)
		for (j=0;j<3;j++)
			C[i][j] = A[i][0]*B[0][j] + A[i][1]*B[1][j] + A[i][2]*B[2][j];
}


/*
 * Multiplica la Matriz A por el escalar a 
 **/
void Mult_Escalar(double a, double A[3][3], double A2[3][3]){
  int i,j;
  for (i = 0; i < 3; i++) {
    for (j = 0; j <3; j++) {
      A2[i][j] = A[i][j] * a;
    }
  }
}

/*Norma de tres valores */
double Norma(double a, double b, double c)
{
  return( sqrt(a*a + b*b + c*c) );
}


/*
 * Calcula la matríz R y t dada una matríz H
 * */
int parametros_de_H( double H[3][3], double MK[16], double MRT[16], double Hi, double Wi)
{
    double u0,v0,w33,f2,f;
    double l1, l2 ,l;
    double A[3][3];
    double A2[3][3];
    double Rp[3][3];
    double R[3][3];

    double RT[3][3];
    double RN[3][3];
	  double Kinv[3][3];
    double vt[3], vc[3];
	  double theta1, theta2, theta3;

	  int i;

    u0 = -Wi/2.0;
    v0 = -Hi/2.0;
    //printf("u0 = %f, v0 = %f\n", u0, v0);

    w33 = (  -H[0][1]*( H[0][0] + H[2][0]*u0 )
                - H[1][1]*( H[1][0] + H[2][0]*v0 )
                - H[2][1]*H[0][0]*u0
                - H[2][1]*H[1][0]*v0  )/(H[2][1]*H[2][0]);

    f2 = w33 - u0*u0 - v0*v0;
    f = 0;
    if (f2 <  0){
        return -1;
    } else{
        f = sqrt( f2 );
    }

	printf("\n\t f^2= %f \n", f2 );
	printf("\n\t f= %f \n", f );

    // double K[3][3] ={{f,0.0,u0},{0.0,f,v0},{0.0,0.0,-1.0}};
    // Kinv = numpy.matrix( [[1.0/f, 0.0, u0/f], [0.0, 1.0/f, v0/f], [0.0, 0.0, -1.0]] )
	Kinv[0][0] = 1.0/f; Kinv[0][1] = 0.0;   Kinv[0][2] = u0/f;
	Kinv[1][0] = 0.0;   Kinv[1][1] = 1.0/f; Kinv[1][2] = v0/f;
	Kinv[2][0] = 0.0;   Kinv[2][1] = 0.0;   Kinv[2][2] = -1.0;

    multiplicaAB(Kinv, H, A);

    l1 = Norma(A[0][0], A[1][0], A[2][0]);
    l2 = Norma(A[0][1], A[1][1], A[2][1]);
    l = (l1+l2)/2.0;

    Mult_Escalar((1.0/l),A,A2);
    vt[0] = A2[0][2];
    vt[1] = A2[1][2];
    vt[2] = A2[2][2];
    CopiaMatriz(A2,Rp);


    Rp[0][2] = Rp[1][0]*Rp[2][1] - Rp[1][1]*Rp[2][0];
    Rp[1][2] = Rp[2][0]*Rp[0][1] - Rp[2][1]*Rp[0][0];
    Rp[2][2] = Rp[0][0]*Rp[1][1] - Rp[0][1]*Rp[1][0];

    CalculaR(Rp, R);

     printf("R = \n");
     for (i = 0; i < 3; i++) {
       for (int j = 0; j < 3; j++)
         printf("%lf ", R[i][j]);
       printf("\n");
     }

	// Cálculo de theta3, theta2, theta1
	// theta1 = atan2( R[2][1], -R[2][0] );
	// theta1 *= 180.0/M_PI;
  //
	// theta2 = acos( R[2][2] );
	// theta2 *= 180.0/M_PI;
  //
	// theta3 = atan2( R[1][2], R[0][2] );
	// theta3 *= 180.0/M_PI;

	// printf( "(theta3, theta2, theta1) = %lf, %lf, %lf\n", theta3, theta2, theta1 );

    // TranspuestaMatriz(R,RT);
    // Mult_Escalar(-1, RT, RN);
    // Mult_Mat_Vector(RN, vt, vc);

    //Inicialización
    for (i = 0; i < 16; i++){
        MK[i] = 0.0;
        MRT[i] = 0.0;
    }


    MK[0]  = f;
    MK[5]  = f;
    MK[8]  = u0;
    MK[9]  = v0;
    MK[10] = 51.0; //**********************************************************//
    MK[11] = -1;
    MK[14] = 20.0; //**********************************************************//
    // MK[14] = 1.0;

    MRT[0 ] = R[0][0]; MRT[1 ] = R[1][0]; MRT[2 ] = R[2][0]; MRT[3 ] = 0.0 ;
    MRT[4 ] = R[0][1]; MRT[5 ] = R[1][1]; MRT[6 ] = R[2][1]; MRT[7 ] = 0.0;
    MRT[8 ] = R[0][2]; MRT[9 ] = R[1][2]; MRT[10] = R[2][2]; MRT[11] = 0.0;
    MRT[12] = vt[0];   MRT[13] = vt[1];   MRT[14] = vt[2];   MRT[15] = 1.0 ;

    
    /*
    printf("        THERE!!!!!!!!!!!!!!!!!     \n");
    printf(" \n MK: \n");
        for (int i = 0; i < 16; ++i)
        {
            if(i%4 == 0) printf("\n");
            printf(" [%d]= %f ", i , MK[i] );
        }

    printf("\n\n MRt: \n");
        for (int i = 0; i < 16; ++i)
        {
            if(i%4 == 0) printf("\n");
            printf(" [%d]= %f ", i , MRT[i] );
        }

    printf("\n DONE -\n");

    */

    return 1;
}

/*
 * Calcula la matríz H (homografía) dado un conjunto de 4
 * puntos de entrada, teniendo como salida la matriz K, R y t.
 * */
int homografia(double P[][2], double p[6][2], double *MK, double *MRT,  double height , double width)
{

    int i,j;

    /*Valores en el modelo */
    double pnor[6][2];
    double Pnor[6][2];
    double vdp[4];
    double vdP[4];
    double R[8][9] ;
    double A[12][9] = {{0}};
    double suma;
    double vh[8] = {0};
    double Hp[3][3]= { {0} };
    double H[3][3]= { {0} };
    double mx, s_x, my, s_y;
    double mu, s_u,mv, s_v;

    normalizar(pnor, p, vdp);
    normalizar(Pnor, P, vdP);

    i = 0;
    while (i < 6){
        j = 2*i;
        A[j][0] = Pnor[i][0];
        A[j][1] = Pnor[i][1];
        A[j][2] = 1.0;
        A[j][6] = -pnor[i][0]*Pnor[i][0];
        A[j][7] = -pnor[i][0]*Pnor[i][1];
        A[j][8] = pnor[i][0];

        j++;
        A[j][3] = Pnor[i][0];
        A[j][4] = Pnor[i][1];
        A[j][5] = 1.0;
        A[j][6] = -pnor[i][1]*Pnor[i][0];
        A[j][7] = -pnor[i][1]*Pnor[i][1];
        A[j][8] = pnor[i][1];

        i++;
    }

	// Tengo que hacer la descomposición QR como en el paper:
    QR_noQ( A, R );

    // for (i = 0; i < 12; i++) {
    //   for (j = 0; j < 9; j++)
    //     printf("%lf ", A[i][j]);
    //   printf("\n");
    // }
    // printf("\n");
    // for (i = 0; i < 8; i++) {
    //   for (j = 0; j < 9; j++)
    //     printf("%lf ", R[i][j]);
    //   printf("\n");
    // }

	/* Backsustitution */
    vh[7] = R[7][8]/R[7][7];

    for( i=6; i>=0; i-- ) {
        suma = 0.0;
        for( j=i+1; j<8; j++ )
            suma += R[i][j] * vh[j];

        vh[i] = ( R[i][8] - suma )/R[i][i];
    }

    //Aquí se tiene H normalizada

    Hp[0][0] = vh[0];
    Hp[0][1] = vh[1];
    Hp[0][2] = vh[2];
    Hp[1][0] = vh[3];
    Hp[1][1] = vh[4];
    Hp[1][2] = vh[5];
    Hp[2][0] = vh[6];
    Hp[2][1] = vh[7];
    Hp[2][2] = 1.0;

    mx = vdP[0];
    s_x = vdP[1];
    my = vdP[2];
    s_y = vdP[3];
    i=0;

    while (i < 3){
        H[i][0] = Hp[i][0]/s_x;
        H[i][1] = Hp[i][1]/s_y;
        H[i][2] = Hp[i][2] - mx*Hp[i][0]/s_x - my*Hp[i][1]/s_y;
        i++;
    }

    mu  = vdp[0];
    s_u = vdp[1];
    mv  = vdp[2];
    s_v = vdp[3];
    Hp[0][0] = (H[0][0] * s_u) + (mu * H[2][0]);
    Hp[1][0] = (H[1][0] * s_v) + (mv * H[2][0]);

    Hp[0][1] = (H[0][1] * s_u) + (mu * H[2][1]);
    Hp[1][1] = (H[1][1] * s_v) + (mv * H[2][1]);

    Hp[0][2] = (H[0][2] * s_u) + (mu * H[2][2]);
    Hp[1][2] = (H[1][2] * s_v) + (mv * H[2][2]);


    i = 0;
    while (i < 3){
        H[0][i] = Hp[0][i];
        H[1][i] = Hp[1][i];
        i ++;
    }

    // printf("H = \n");
    // for (i = 0; i < 3; i++) {
    //   for (j = 0; j < 3; j++)
    //     printf("%lf ", H[i][j]);
    //   printf("\n");
    // }

    i = parametros_de_H(H, MK, MRT, height, width);

	return i;
}

int find_TRIANGLES( MATRIX *IN , POINT* TRI1, POINT* TRI2, int* OBJCOUNT, double *MK, double *MRT, int DBG) 
{
	//printf("Finding triangles now... \n");

	//MATRIX *IMG, *FIL;
	//IMG = matrix_allocate( IN->rows, IN->cols, sizeof(byte) );

	if(DBG)write_pnm( IN, "00_TEST.pgm" );

	if ( IN == NULL ) {
		fprintf( stderr, "ERROR in IMG from camera \n" );
		return -1;
	}
	if ( IN->type != 5 ) {
		fprintf( stderr, "ERROR Bad format. Image must be in b/w \n");
		return -1;
	}
	
	//2 - Smoothing (2 Pass)
		//int **fil;
		//FIL = matrix_allocate( 3, 3, sizeof( int ) );
		//if ( FIL==NULL ) {
		//	fprintf( stderr, "ERROR allocating memory\n" );
		//	return -2;
		//}
		//Smooth Window
		//fil = (int **)FIL->ptr;
		//fil[0][0] = 1; fil[0][1] = 1; fil[0][2] = 1;
		//fil[1][0] = 1; fil[1][1] = 1; fil[1][2] = 1;
		//fil[2][0] = 1; fil[2][1] = 1; fil[2][2] = 1;

		convolution( IN, FIL, 9 , IMG );	//1st Pass
							if(DBG)write_pnm( IMG, "1_convol1.pgm" );

		//memcpy( IN, IMG, sizeof(MATRIX) );
		//IMG = convolution( IN, FIL, 9 );	//2nd Pass
				//write_pnm( IMG, "convol2.pgm" );

	
	//3 - Binarize IMG global threshold 
		umbral( IMG, 30 );
							if(DBG)write_pnm( IMG, "2_threshold.pgm" );

	//4 - Opening / Erotion->Dilatation
		//int Umbral = 13;
		//Opening
			//Erosion
			negate( IMG );
			chamfer34( IMG ); 
			umbral( IMG , Umbral );
			negate( IMG );
							if(DBG)write_pnm( IMG, "3_EROTION.pgm" );
			//Dilatation
			chamfer34( IMG );
			umbral( IMG , Umbral );
							if(DBG)write_pnm( IMG, "4_DILATATION.pgm" );
			cleanBorder( IMG , Umbral );
							if(DBG)write_pnm( IMG, "5_OPENING.pgm" );

	//5 - Extract triangles (Fill)
							if(DBG)printf("#Looking for objects... \n");
		//int X,Y;
		//int OBJcount = 0;
		//int OBJ[5];
		//int COLOR;
		
		//Look for objects to fill them
		OBJcount = 0;					
		while(1)
		{
			findOBJ(IMG, &X, &Y, 0);
			if(X > -1){ //Valid Point
				//printf("#Found object at: %d , %d \n", X,Y);
				COLOR = FILLTONE + OBJcount * CINC;
				
				push(X,Y);
				do_fill(IMG, OBJ , 0 , COLOR );	//Fill 0's with COLOR

				OBJcount+=1;
			}
			else break;
		}
		*OBJCOUNT = OBJcount;	//return Objects found

		if(DBG)printf("# \t [%d] Objects found! \n", OBJcount);

		if( OBJcount != 2 ){
			//printf("# \t [%d] Objects found! \n", OBJcount); 
			return( -3 );
		}


							if(DBG)write_pnm( IMG, "6_Segmentation.pgm" );		


	//Save each Scene vertex in a list
	//DPOINT* PointS = malloc( sizeof(DPOINT) * 3 * OBJcount );
 
	//6 - Compute vertices of each triangle
							if(DBG)printf("# \n" );

		//Allocate memory enough for ANY perimeter segment
		//POINT *perimeter = malloc( sizeof(POINT) * 2000 );
		//Allocate memory for each triangle vertices
		//POINT* vertices = malloc( sizeof(POINT) * 3 );
		//DPOINT* dv = malloc( sizeof(DPOINT) * 3 );

		for (int i = 0; i < 2; ++i)//OBJcount; ++i)
		{	


			COLOR = FILLTONE + i * CINC;

			//a) Find first pixel of OBJColor
			findOBJ(IMG, &X, &Y, COLOR); //Find first vertex
			if(X > -1){ //Valid Point
							if(DBG)printf("\n\t ### Object [%d] ###\n\n",i+1);

				//Get Perimeter
				push(X,Y);
				do_fill(IMG, OBJ , COLOR , 1 );	//Fill COLOR's with 1's
						//write_pnm( IMG, "Fill.pgm" );		
				dilation_4neig_inplace_obj( IMG , OBJ, 1 );
						//write_pnm( IMG, "perimetroD.pgm" );		

				//Convert perimeter
				//byte *pimg;
				for(int k=OBJ[INICIOY]; k<=OBJ[FINY]; k++ ) {
					pimg = (byte *)IMG->ptr[k] + OBJ[INICIOX];
					for(int p=OBJ[INICIOX]; p<=OBJ[FINX]; p++ ) {
						if ( *pimg == 254 ) {
							*pimg = 200;
						}
						pimg++;
					}
				}
				//write_pnm( IMG, "perimetroE.pgm" );		

		
				//printf("\n Size of POINT:: %d \n", sizeof(POINT) );
				
				Counter = 0;

				int perR = follow_perimeter( IMG , Y, X , 200,    &Counter,   perimeter);
				if(perR < 0) return -15;
				//printf("#Counter: %d \n",Counter);
								if(DBG)write_pnm( IMG, "8_Perimeter.pgm" );		
				/*
				printf("\t OUTSIDE: Pixels of segment\n");
				for (int k = 0; k <= Counter ; k++)
				{
					printf(" %d,%d \t", perimeter[k].x, perimeter[k].y );
				}
				//*/

				//Extract triangles
				
				extrae_triangulo( perimeter , Counter , vertices, dv );
				
				//printf("#Triangle extracted...\n");

				if(DBG)printf("#\t     %d,%d \t %d,%d \t %d,%d \t#NEW\n", vertices[0].x , vertices[0].y,
																vertices[1].x , vertices[1].y,
																vertices[2].x , vertices[2].y);

				if(DBG)printf("#\t %.02f,%.02f \t %.02f,%.02f \t %.02f,%.02f \t#FLOAT\n", dv[0].x , dv[0].y,
																dv[1].x , dv[1].y,
																dv[2].x , dv[2].y);
										if(DBG)write_pnm( IMG, "10_FINAL.pgm" );		

				//Save vertices in list
				PointS[ 3*i ].x = dv[0].x;		PointS[ 3*i ].y = dv[0].y;  
				PointS[3*i+1].x = dv[1].x;		PointS[3*i+1].y = dv[1].y;
				PointS[3*i+2].x = dv[2].x;		PointS[3*i+2].y = dv[2].y;

			}
		}

		if(DBG){
			printf("\n\n#\t Scene [P] Vertex List:\n");
			for (int i = 0; i < 2; ++i)//OBJcount; ++i)
			{
				printf("#\t %.02f,%.02f \t %.02f,%.02f \t %.02f,%.02f \t\t #TRI:%d \n", PointS[ 3*i ].x , PointS[ 3*i ].y,
																					 PointS[3*i+1].x , PointS[3*i+1].y,
																					 PointS[3*i+2].x , PointS[3*i+2].y, i+1);
					
			}
		}

		//Copy points to structure
		if(DBG) printf("\n\n#\t Saving points to SCENE \n");
		

		///*
		TRI1[0].x = PointS[0].x; TRI1[0].y = PointS[0].y;
		TRI1[1].x = PointS[1].x; TRI1[1].y = PointS[1].y;
		TRI1[2].x = PointS[2].x; TRI1[2].y = PointS[2].y;
		
		TRI2[0].x = PointS[3].x; TRI2[0].y = PointS[3].y;
		TRI2[1].x = PointS[4].x; TRI2[1].y = PointS[4].y;
		TRI2[2].x = PointS[5].x; TRI2[2].y = PointS[5].y;
		//*/

		
		//Compute Lambda matrix for ORDER
		if(DBG)printf("\n\n#\t Computing Lambda matrix for ORDER: ");
		
		int order[6] = {0,0,0, 0,0,0};
		getLambdaOrder( PointS , 6, order);

		if(DBG)printf("#New order..\n");


		//Inverting y's of points 
		for (int i = 0; i < 6; ++i)
		{
			PointS[i].y = 480 - PointS[i].y;
		}

		/*
		TRI1[0].x = PointS[0].x; TRI1[0].y = PointS[0].y;
		TRI1[1].x = PointS[1].x; TRI1[1].y = PointS[1].y;
		TRI1[2].x = PointS[2].x; TRI1[2].y = PointS[2].y;
		
		TRI2[0].x = PointS[3].x; TRI2[0].y = PointS[3].y;
		TRI2[1].x = PointS[4].x; TRI2[1].y = PointS[4].y;
		TRI2[2].x = PointS[5].x; TRI2[2].y = PointS[5].y;
		//*/

		if(DBG){
			printf("\n#\t INVERTED Scene [P] Vertex List:\n");
			for (int i = 0; i < OBJcount; ++i)
			{
				printf("#\t %.02f,%.02f \t %.02f,%.02f \t %.02f,%.02f \t\t #TRI:%d \n", PointS[ 3*i ].x , PointS[ 3*i ].y,
																					 PointS[3*i+1].x , PointS[3*i+1].y,
																					 PointS[3*i+2].x , PointS[3*i+2].y, i+1);
					
			}
		}
	//Compute Homography
		
		//Img Points
		DPOINT p[6], pORG[6];
		DPOINT PPP[6];
		
		//MARKER with order
		p[0].x = 0;		p[0].y = 255;
		p[1].x = 0;		p[1].y = 0;
		p[2].x = 30;	p[2].y = 100;
		p[3].x = 130;	p[3].y = 90;
		p[4].x = 255;	p[4].y = 0;
		p[5].x = 255;	p[5].y = 255;

		//MARKER ORIGINAL
		pORG[0].x = 0;		pORG[0].y = 0;
		pORG[1].x = 255;	pORG[1].y = 0;
		pORG[2].x = 255;	pORG[2].y = 255;
		pORG[3].x = 0;		pORG[3].y = 255;
		pORG[4].x = 30;		pORG[4].y = 100;
		pORG[5].x = 130;	pORG[5].y = 90;	

		//MARKER ORIGINAL
		PPP[0].x = 0;		PPP[0].y = 0;
		PPP[1].x = 255;		PPP[1].y = 0;
		PPP[2].x = 130;		PPP[2].y = 90;
		PPP[3].x = 0;		PPP[3].y = 255;
		PPP[4].x = 255;		PPP[4].y = 255;
		PPP[5].x = 30;		PPP[5].y = 100;	

		//ORDER OF POINTS
		//short order[6] = {1,0,5,4,3,2}; //IMG1
		//short order[6] = {0,2,5,4,3,1};	//IMG2
		//short order[6] = {0,2,5,4,3,1};	//IMG3
		//short order[6] = {1,0,5,4,3,2};	//IMG4
		//short order[6] = {0,2,5,4,3,1};	//IMG5

		if(DBG){
			printf("\n# []Order \n# ");
			for (int i = 0; i < 6; ++i)
				printf(" %d",order[i]);

			//Print relation of points
			printf("\n\n");
			for (int i = 0; i < 6; ++i)
				printf(" \t %d - [%.02f,%.02f] -> [%.04f,%.04f] \n", i ,p[i].x,p[i].y,   PointS[order[i]].x, PointS[order[i]].y  );
		}

		double Modelo[6][2] = {{0.,255.},{0.,0.},{30.,100.},{130.,90.},{255.,0.},{255.,255.}};
    	double Puntos[6][2];

    	Puntos[0][0] = PointS[order[0]].x;		Puntos[0][1] = PointS[order[0]].y;
    	Puntos[1][0] = PointS[order[1]].x;		Puntos[1][1] = PointS[order[1]].y;
    	Puntos[2][0] = PointS[order[2]].x;		Puntos[2][1] = PointS[order[2]].y;
    	Puntos[3][0] = PointS[order[3]].x;		Puntos[3][1] = PointS[order[3]].y;
    	Puntos[4][0] = PointS[order[4]].x;		Puntos[4][1] = PointS[order[4]].y;
    	Puntos[5][0] = PointS[order[5]].x;		Puntos[5][1] = PointS[order[5]].y;


    	//printf("                PUNTOS            \n");
    	//for (int i = 0; i < 6; ++i)
    	//{
    	//	printf("  %f,%f \n", Puntos[i][0], Puntos[i][1] );
    	//}


    
		homografia( Modelo, Puntos, MK,  MRT,  480 , 640 );

/*
		//Compute median and variance of P (scene points)
			medianX = 0;
			medianY = 0;
			for (int i = 0; i < 6; ++i)
			{
				medianX += PointS[ i ].x;
				medianY += PointS[ i ].y;
			}
			medianX /= 6;
			medianY /= 6;
					if(DBG) printf("\n\n# Median of [PointS]: %f , %f \n", medianX, medianY);

			varX = varY = 0;
			for (int i = 0; i < 6; ++i)
			{
				varX += (PointS[i].x - medianX) * (PointS[i].x - medianX);
				varY += (PointS[i].y - medianY) * (PointS[i].y - medianY);
			}
			varX /= 6;
			varY /= 6;
					if(DBG) printf("#variance of [PointS]: %f , %f \n", varX, varY);

			//Normalize vertices in image
			for (int i = 0; i < 6; ++i)
			{
				PointS[i].x = (PointS[i].x - medianX ) /varX;
				PointS[i].y = (PointS[i].y - medianY ) /varY;
			}
			//Build T1^-1 (inverse)
			T1[0][0] = varX;	T1[0][1] = 0; 		T1[0][2] = medianX;
			T1[1][0] = 0;		T1[1][1] = varY; 	T1[1][2] = medianY;
			T1[2][0] = 0;		T1[2][1] = 0; 		T1[2][2] = 1;
			
				if(DBG){
					printf("\n# T1 \n");
					for (int i = 0; i < 3; ++i)
					{
						for (int j = 0; j < 3; ++j)
						{
							printf(" %f ", T1[i][j] );
						}
						printf("\n");
					}

					printf("\n\t#Normalized [PointS] Vertex List:\n");
					for (int i = 0; i < OBJcount; ++i)
					{
						printf("#\t %.04f,%.04f \t %.04f,%.04f \t %.04f,%.04f \t #TRI:%d \n", PointS[ 3*i ].x , PointS[ 3*i ].y,
																							 PointS[3*i+1].x , PointS[3*i+1].y,
																							 PointS[3*i+2].x , PointS[3*i+2].y, i+1);
					}
				}

		//Compute median and variance of p (marker/img points)
			medianX = 0;
			medianY = 0;
			for (int i = 0; i < 6; ++i)
			{
				medianX += p[ i ].x;
				medianY += p[ i ].y;
			}
			medianX /= 6;
			medianY /= 6;
					if(DBG) printf("\n\n#  Median of [p]: %f , %f \n", medianX, medianY);

			varX = varY = 0;
			for (int i = 0; i < 6; ++i)
			{
				varX += (p[i].x - medianX) * (p[i].x - medianX);
				varY += (p[i].y - medianY) * (p[i].y - medianY);
			}
			varX /= 6;
			varY /= 6;
					if(DBG) printf(" Variance of [p]: %f , %f \n", varX, varY);

			//Normalize vertices in image
			for (int i = 0; i < 6; ++i)
			{
				p[i].x = (p[i].x - medianX)/varX;
				p[i].y = (p[i].y - medianY)/varY;
			}
			
			//Build T2
			T2[0][0] = 1/varX;	T2[0][1] = 0; 		T2[0][2] = -medianX/varX;
			T2[1][0] = 0;		T2[1][1] = 1/varY; 	T2[1][2] = -medianY/varY;
			T2[2][0] = 0;		T2[2][1] = 0; 		T2[2][2] = 1;
			
			if(DBG){
				printf("\n# T2 \n");
				for (int i = 0; i < 3; ++i)
				{
					for (int j = 0; j < 3; ++j)
					{
						printf(" %f ", T2[i][j] );
					}
					printf("\n");
				}

				printf("\n\n#\tNormalized [p] Vertex List:\n");
				for (int i = 0; i < OBJcount; ++i)
				{
					printf("#\t %.04f,%.04f \t %.04f,%.04f \t %.04f,%.04f \t #TRI:%d \n", p[ 3*i ].x , p[ 3*i ].y,
																						 p[3*i+1].x , p[3*i+1].y,
																						 p[3*i+2].x , p[3*i+2].y, i+1);
				}
			}

		//Build Matrix B = [A|b]

			if(DBG){
				for (int i = 0; i < 12; ++i)
				{
					for (int j = 0; j < 9; ++j)
					{
						B[i][j] = -1; 
					}
				}

				
				//Print B
				printf("B=\n");
				for (int i = 0; i < 12; ++i)
				{
					for (int j = 0; j < 9; ++j)
					{
						printf("  [%d,%d]= %.04f",i,j, B[i][j] ); 
					}
					printf("\n");
				}
			}


			if(DBG)printf("\n# >>Building matrix B = [A|b] \n");
		for (int k = 0; k < 6; ++k)
		{	
			//Build A
			short i;
			//(2*k)-th row
			i = 2*k;
			B[i][0] = p[k].x;	B[i][1] = p[k].y;	B[i][2] = 1;  
			B[i][3] = 0;	B[i][4] = 0;	B[i][5] = 0;  
			B[i][6] = -p[k].x * PointS[order[k]].x;	B[i][7] = -p[k].y * PointS[order[k]].x;
				//build b
				B[i][8] = PointS[order[k]].x;
			
			//(2*k+1)-th row
			i = 2*k+1;
			B[i][0] = 0;	B[i][1] = 0;	B[i][2] = 0;  
			B[i][3] = p[k].x;	B[i][4] = p[k].y;	B[i][5] = 1;  
			B[i][6] = -p[k].x * PointS[order[k]].y;	B[i][7] = -p[k].y * PointS[order[k]].y;
				//build b
				B[i][8] = PointS[order[k]].y;
		}

				if(DBG){
					printf("#B=\n");
					for (int i = 0; i < 12; ++i)
					{
						for (int j = 0; j < 9; ++j)
						{
							//printf("  [%d,%d]= %.04f",i,j, B[i][j] ); 
							printf(" %.04f \t", B[i][j] ); 
						}
						printf("\n");
					}
				}


		//QR Factorization
				if(DBG)printf("\n# >>QR Factorization\n");

			//double * H;
			//H = (double *)malloc( 12*sizeof(double) );
			//for (int i = 0; i < 12; ++i)
			//	printf("[%d]=%.4f \t",i,H[i] );

			descomposicionQR( B, 12 , 9 , R, H );

			if(DBG){
				printf("\t# R[9x9]=\n");
				for (int i = 0; i < 9; ++i)
				{
					for (int j = 0; j < 9; ++j)
					{
						//printf("  [%d,%d]= %.04f",i,j, B[i][j] ); 
						printf(" %.04f \t", R[i][j] ); 
					}
					printf("\n");
				}
			}

		//Compute H
			if(DBG) printf("\n# >>Solving by backward substitution\n");
		resuelve( R , 8 , H);

			if(DBG){
				printf("\t# H[]=\n");
				for (int i = 0; i < 9; ++i)
					(i==8)?printf("|| [%d]=%.4f ",i,H[i] ):printf(" [%d]=%.4f ",i,H[i] );
			}

		//Normalize Homography
			
				//Print blank H
				if(DBG){
					printf("\n   [ H ] \n");
					for (int i = 0; i < 3; ++i)
					{
						for (int j = 0; j < 3; ++j)
						{
							printf(" %f ", Hn[i][j] );
						}
						printf("\n");
					}
				}


			//Copy homography array to matrix
			Hn[0][0] = H[0];	Hn[0][1] = H[1];	Hn[0][2] = H[2];
			Hn[1][0] = H[3];	Hn[1][1] = H[4]; 	Hn[1][2] = H[5];
			Hn[2][0] = H[6];	Hn[2][1] = H[7]; 	Hn[2][2] = 1;

				if(DBG){
					printf("\n#   [ H ] \n");
					for (int i = 0; i < 3; ++i)
					{
						for (int j = 0; j < 3; ++j)
						{
							printf(" %f ", Hn[i][j] );
						}
						printf("\n");
					}
				}

			matrix_mul( T1 , Hn , r);
			matrix_mul( r , T2 , Hn );

				if(DBG){
					printf("\n#   [ H_norm ] \n");
					for (int i = 0; i < 3; ++i)
					{
						for (int j = 0; j < 3; ++j)
						{
							printf(" %f ", Hn[i][j] );
						}
						printf("\n");
					}
				}

			//Copy back homography array from matrix
			H[0] = Hn[0][0];	H[1] = Hn[0][1];	H[2] = Hn[0][2];
			H[3] = Hn[1][0];	H[4] = Hn[1][1]; 	H[5] = Hn[1][2];
			H[6] = Hn[2][0];	H[7] = Hn[2][1]; 	H[8] = Hn[2][2];

			//Project original points
						if(DBG)printf("\n# INVERTED HOMOGRAPHY rounded points\n ");
					for (int i = 0; i < 6; ++i)
					{
						//printf("Point = %f , %f \n", pORG[i].x, pORG[i].y);
						PX[0][0] = PPP[i].x; 
						PX[1][0] = PPP[i].y;  
						PX[2][0] = 1;
						//PX[3][0] = 1;

						matrix_MUL( Hn ,3,3, PX, 3,1, pRes );

						//Normalize
						for (int i = 0; i < 3; ++i)
							pRes[i][0] = pRes[i][0]/pRes[2][0];

						//printf("\t (%d) [ %.02f , %.02f , %.02f  ] \n", i , pRes[0][0], pRes[1][0], pRes[2][0] );
						int X = (int)floor(pRes[0][0]+0.5);
						int Y = (int)floor((pRes[1][0])+0.5);
						//printf("\t %d,%d \n", X , Y );


						//Copy to SCENE /////////////////////////////////////7
						if(i<3){
							TRI1[i].x = X; TRI1[i].y = Y;
						}
						else{
							TRI2[i-3].x = X; TRI2[i-3].y = Y;
						}
					}

				printf("  Calling help... \n");
				parametros_de_H( Hn ,  MK , MRT , 480, 640);
					
/*

				//Compute f
				double u = 640 / 2.0; // 0 **
				double v = 480 / 2.0;
				double ff = ( u * ( H[1]*H[6] + H[7]*H[0] - u*H[7]*H[6] ) 
							+ v * ( H[4]*H[6] + H[7]*H[3] - v*H[7]*H[6] )
							- H[1]*H[0] - H[4]*H[3] ) 
							/ H[7]*H[6] ;
				double f = sqrt(ff);

						if(DBG){
							printf("\n\t f^2= %f \n", ff );
							printf("\n\t f= %f \n", f );
						}

						/*
						double w33 = (  -Hn[0][1]*( Hn[0][0] + Hn[2][0]*u )
						                - Hn[1][1]*( Hn[1][0] + Hn[2][0]*v )
						                - Hn[2][1]*Hn[0][0]*u
						                - Hn[2][1]*Hn[1][0]*v  )/(Hn[2][1]*Hn[2][0]);

					    ff = w33 - u*u - v*v;
					    f = 0;
					    


					    if(DBG){
							printf("\n\t f^2= %f \n", ff );
							printf("\n\t f= %f \n", f );
						}

						if (ff <  0){
					        return -1;
					    } else{
					        f = sqrt( ff );
					    }

						//////












			//Compute Matrix K
			
			//Build matrix K
				K[0][0] = f;	K[0][1] = 0;	K[0][2] = -u;
				K[1][0] = 0;	K[1][1] = f; 	K[1][2] = -v;
				K[2][0] = 0;	K[2][1] = 0; 	K[2][2] = -1.0;

			//Build matrix K^-1
				Ki[0][0] = 1/f;	Ki[0][1] = 0;	Ki[0][2] = -u/f;
				Ki[1][0] = 0;	Ki[1][1] = 1/f; Ki[1][2] = -v/f;
				Ki[2][0] = 0;	Ki[2][1] = 0; 	Ki[2][2] = -1.0;
		
				if(DBG){
					printf("\n K[]= \n");
					for (int i = 0; i < 3; ++i)
					{
						for (int j = 0; j < 3; ++j)
							printf(" %f ", K[i][j] );
						printf("\n");
					}	

					printf("\n K^-1[]= \n");
					for (int i = 0; i < 3; ++i)
					{
						for (int j = 0; j < 3; ++j)
							printf(" %f ", Ki[i][j] );
						printf("\n");
					}
				}

		//Computing G = K^-1 * H	
			matrix_mul( Ki , Hn , G);

				if(DBG){
					printf("\n G = K^-1 * H = \n");
					for (int i = 0; i < 3; ++i)
					{
						for (int j = 0; j < 3; ++j)
						{
							printf(" %f ", G[i][j] );
						}
						printf("\n");
					}
				}

	//Computing Lambda1 and Lamda2
		double Lambda1 = sqrt( G[0][0]*G[0][0] + G[1][0]*G[1][0] + G[2][0]*G[2][0] );
		double Lambda2 = sqrt( G[0][1]*G[0][1] + G[1][1]*G[1][1] + G[2][1]*G[2][1] );
		double Lambda  = (Lambda1 + Lambda2) / 2.0;
				if(DBG) printf("\n\t >>Lambda= %f \n", Lambda );

	//Saving t
			double t [3];
			t[0] = G[0][2]/Lambda;
			t[1] = G[1][2]/Lambda;
			t[2] = G[2][2]/Lambda;

			if(DBG){
				printf("\n\t t[]= \n");
				for (int i = 0; i < 3; ++i)
					printf("\t  %f \n", t[i] );
			}

/*
						double G2[3][3];
					    double l1, l2 ,l;


						multiplicaAB( Ki , Hn , G);
								if(DBG){
									printf("\n G = K^-1 * H = \n");
									for (int i = 0; i < 3; ++i)
									{
										for (int j = 0; j < 3; ++j)
										{
											printf(" %f ", G[i][j] );
										}
										printf("\n");
									}
								}
					    l1 = Norma(G[0][0], G[1][0], G[2][0]);
					    l2 = Norma(G[0][1], G[1][1], G[2][1]);
					    l = (l1+l2)/2.0;

					    Mult_Escalar((1.0/l), G , G2);

					    	if(DBG){
									printf("\n G2222 = K^-1 * H = \n");
									for (int i = 0; i < 3; ++i)
									{
										for (int j = 0; j < 3; ++j)
										{
											printf(" %f ", G2[i][j] );
										}
										printf("\n");
									}
								}
					    t[0] = G2[0][2];
					    t[1] = G2[1][2];
					    t[2] = G2[2][2];


					    if(DBG){
							printf("\n\t t[]= \n");
							for (int i = 0; i < 3; ++i)
								printf("\t  %f \n", t[i] );
						}
/////


	//Computing r1 x r2
		//R1							//R2							// R1 x R2
		Rd[0][0] = G[0][0]/Lambda;		Rd[0][1] = G[0][1]/Lambda;		Rd[0][2] = (G[1][0]*G[2][1] - G[2][0]*G[1][1])/Lambda;  
		Rd[1][0] = G[1][0]/Lambda;		Rd[1][1] = G[1][1]/Lambda;		Rd[1][2] = (G[2][0]*G[0][1] - G[0][0]*G[2][1])/Lambda;
		Rd[2][0] = G[2][0]/Lambda;		Rd[2][1] = G[2][1]/Lambda;		Rd[2][2] = (G[0][0]*G[1][1] - G[1][0]*G[0][1])/Lambda;

			if(DBG){
				printf("\n R' = \n");
				for (int i = 0; i < 3; ++i)
				{
					for (int j = 0; j < 3; ++j)
					{
						printf(" %f ", Rd[i][j] );
					}
					printf("\n");
				}
			}

		//Compute R'= UDV^T 
			double* dummy = (double*) malloc(3 * sizeof(double));                  
			int err = Singular_Value_Decomposition( Rd , 3, 3, UU, *D, VT, dummy);		
			free(dummy);                                                     
	     	if (err < 0) printf(" Failed to converge\n"); 
	     	       	
	     	       	if( DBG ){
		     	       	printf("\n [D]= \n");
						for (int i= 0; i< 3; i++ )
							printf(" %lf ", D[ i ][0] );

						printf("\n\n [U]= \n");
						for (int i= 0; i< 3; i++ ) {
							for (int j= 0; j< 3; j++ )
								printf("%lf ", UU[i][j] );
							printf("\n");
						}

						printf("\n [V]= \n");
						for (int i= 0; i< 3; i++ ) {
							for (int j= 0; j< 3; j++ )
								printf("%lf ", VT[i][j] );
							printf("\n");
						}       
					}         


		//Transpose V
			double aux;
			aux = VT[0][1]; 	VT[0][1] = VT[1][0]; 	VT[1][0] = aux;
			aux = VT[0][2]; 	VT[0][2] = VT[2][0]; 	VT[2][0] = aux;
			aux = VT[1][2]; 	VT[1][2] = VT[2][1]; 	VT[2][1] = aux;

				if(DBG){
					printf("\n [V^T]= \n");
					for (int i= 0; i< 3; i++ ) {
						for (int j= 0; j< 3; j++ )
							printf("%lf ", VT[i][j] );
						printf("\n");
					}
				}

		//Compute R = U*V^T
		matrix_mul( UU , VT , Rd );

			if( DBG ){
					printf("\n\n ------------------- \n");
					printf("\n R = U * V^T \n");
					for (int i = 0; i < 3; ++i)
					{
						for (int j = 0; j < 3; ++j)
						{
							printf(" %f ", Rd[i][j] );
						}
						printf("\n");
					}

					printf("\n\t t[]= \n");
					for (int i = 0; i < 3; ++i)
							printf("\t  %f \n", t[i] );

					printf("************\n");
					
					printf("\n K[]= \n");
					for (int i = 0; i < 3; ++i)
					{
						for (int j = 0; j < 3; ++j)
						{
							printf(" %f ", K[i][j] );
						}
						printf("\n");
					}
			}

		//*******
		//Building augmented matrix [R|t]
		
		//Build matrix [R|t]
		for (int i = 0; i < 3; ++i)
		{	
			int j;
			for (j = 0; j < 3; ++j)
				RT[i][j] = Rd[i][j];
			
			RT[i][j] = t[i];
		}

		static int Z_NEAR = 4;
		static int Z_FAR = -4;

		MK[0]  = f;
	    MK[5]  = f;
	    MK[8]  = -u;
	    MK[9]  = -v;
	    MK[10] = 51;	//Z_NEAR + Z_FAR; //**********************************************************
	    MK[11] = -1.0;
	    MK[14] = 20; 	//Z_NEAR * Z_FAR; //********************************************************** 
    

		MRT[0 ] = RT[0][0]; MRT[1 ] = RT[1][0]; MRT[2 ] = RT[2][0]; MRT[3 ] = 0.0 ;
	    MRT[4 ] = RT[0][1]; MRT[5 ] = RT[1][1]; MRT[6 ] = RT[2][1]; MRT[7 ] = 0.0;
	    MRT[8 ] = RT[0][2]; MRT[9 ] = RT[1][2]; MRT[10] = RT[2][2]; MRT[11] = 0.0;
	    MRT[12] = t[0];   	MRT[13] = t[1];		MRT[14] = t[2];   	MRT[15] = 1.0 ;
    

		
				if(DBG){
					printf("\n [R|t] \n");
					for (int i = 0; i < 3; ++i)
					{
						for (int j = 0; j < 4; ++j)
						{
							printf(" %f ", RT[i][j] );
						}
						printf("\n");
					}
				}

		matrix_MUL( K ,3,3, RT, 3,4, M );

				if(DBG){
					printf("\n M = K[R|t] \n");
					for (int i = 0; i < 3; ++i)
					{
						for (int j = 0; j < 4; ++j)
						{
							printf(" %f ", M[i][j] );
						}
						printf("\n");
					}
				}


		//////
		//Converting points KRT			


			if(DBG)printf("\n Projecting  KRT original points...\n ");
			for (int i = 0; i < 6; ++i)
			{
				//printf("Point = %f ,%f \n", pORG[i].x, pORG[i].y);
				PX[0][0] = PPP[i].x; 
				PX[1][0] = PPP[i].y;  
				PX[2][0] = 0;
				PX[3][0] = 1;

				matrix_MUL( M ,3,4, PX, 4,1, pRes );

				//Normalize
				for (int i = 0; i < 3; ++i)
					pRes[i][0] = pRes[i][0]/pRes[2][0];

				//printf("\t (%d) [ %.02f , %.02f , %.02f ] \n", i , pRes[0][0], pRes[1][0], pRes[2][0] );

				int X = (int)floor(pRes[0][0]+0.5);
				int Y = (int)floor((pRes[1][0])+0.5);
				//printf("\t %d,%d \n", X , Y );


				//Copy to SCENE /////////////////////////////////////7
				//if(i<3){
				//	TRI1[i].x = X; TRI1[i].y = Y;
				//}
				//else{
				//	TRI2[i-3].x = X; TRI2[i-3].y = Y;
				//}


			}





*/




















	//free(vertices); free(dv);
	//free(perimeter);
	//free(PointS);

	//matrix_free( IMG );
	//matrix_free( FIL );

	return 0;
}










// Matrix Multiplication
//
void matrix_MUL(double ** A,int m, int n, double ** B, int p, int q, double ** C){
	if (n != p)
    printf("The matrices can't be multiplied with each other.\n");	

	int i, j, k;
	double sum = 0;

	for (i = 0; i < m; i++) {
      for (j = 0; j < q; j++) {
        for (k = 0; k < p; k++) {
          sum = sum + A[i][k] * B[k][j];
        }
 
        C[i][j] = sum;
        sum = 0;
      }
    }

}

// 3x3 Matrix Multiplication
//
void matrix_mul(double ** A,double ** B,double ** C){

   for (int i = 0; i < 3; i++) {
      for (int j = 0; j < 3; j++) {
         C[i][j] = 0;
         for (int k = 0; k < 3; k++)
            C[i][j] = C[i][j] + A[i][k] * B[k][j];
      }
   }
}


/**
  Subrutina que calcula descomposición QR
  A = QR, A es de tamaño m x n
  No retorna el valor de  Q
**/
void descomposicionQR( double **A, int m, int n, double **R, double *vec )
{
	int i, j, k;
	double sum, tmp;

	for( k=0; k<n; k++ ) {

		sum = 0.0;
		for( i=0; i<m; i++ ) {
			tmp = A[i][k];
			sum += tmp * tmp;
		}
		sum = sqrt( sum );
		R[k][k] = sum;
		for( i=0; i<m; i++ )
			vec[i] = A[i][k] / sum;

		for( j=k+1; j<n; j++ ) {
			sum = 0.0;
			for( i=0; i<m; i++ ) 
				sum += vec[i] * A[i][j];

			R[k][j] = sum;

			for( i=0; i<m; i++ ) 
				A[i][j] -= vec[i] * sum;
		}
	}
}


/**
 Si se tiene el sistema Ax = b, A es de tamaño n x n, 
se realiza la matriz aumentada [A|b] y se realiza
la descomposición QR, se obtiene Q[ R | Q^T b ]
Esta rutina obtiene x a partir de la matriz Rp = [ R | Q^T b ]
**/
void resuelve( double **Rp, int n, double *x )
{
	int i, j;
	double suma;

	j = n-1;

	x[j] = Rp[j][n] / Rp[j][j];

	for( i=n-2; i>=0; i-- ) {
		suma = 0.0;
		for( j=i+1; j<n; j++ )
			suma += Rp[i][j] * x[j];

		x[i] = (Rp[i][n] - suma)/Rp[i][i];
	}
}








// --------------------
// 
void drawPoint( MATRIX *IMG , int X, int Y , int R , int C) 
{
	byte *px;
	for (int i = Y-R; i < Y+R; i++)
	{	
		px = (byte *) IMG-> ptr[i];
		for (int j = X-R; j < X+R; j++){
			*px++ = C;
		}
	}
}



// --------------------
// Find first OBJ pixel in IMG
void findOBJ( const MATRIX *IMG , int* X, int* Y , int COLOR) 
{
	byte *px;
	for (int i = 0; i < IMG-> rows; i++)
	{	
		px = (byte *) IMG-> ptr[i];
		for (int j = 0; j < IMG-> cols; j++){
			if((*px) == COLOR){
				//printf("[%d,%d]= %d  ",i,j,*px);
				*Y=i;
				*X=j;
				return;
			}
			px++;
		}
	}
	*Y=	*X= -1; //No OBJ found!
}







